package com.example.transformation.executor;

import java.io.File;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.BasicDiagnostic;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.emf.examples.extlibrary.EXTLibraryPackage;
import org.eclipse.m2m.qvt.oml.BasicModelExtent;
import org.eclipse.m2m.qvt.oml.ExecutionContextImpl;
import org.eclipse.m2m.qvt.oml.ExecutionDiagnostic;
import org.eclipse.m2m.qvt.oml.ModelExtent;
import org.eclipse.m2m.qvt.oml.TransformationExecutor;
import org.eclipse.m2m.qvt.oml.util.Log;
import org.eclipse.m2m.qvt.oml.util.WriterLog;
import org.eclipse.uml2.uml.UMLPackage;

import org.osgi.framework.Bundle;


public class LibraryExecutor {
	
	String libPath;
	String umlPath;
	String transPath;
	
	
	public LibraryExecutor(String libPath,	String umlPath) {
		
		this.libPath = libPath;
		this.umlPath = umlPath;
		
		Bundle bundle = Platform.getBundle("com.example.tranformation");
		URL fileURL = bundle.getEntry("transforms/ExtLibraryTransformation.qvto");
		
		try {
		    File file = new File(FileLocator.resolve(fileURL).toURI());
		    transPath = file.getCanonicalPath();
		} catch (URISyntaxException e1) {
		    e1.printStackTrace();
		} catch (IOException e1) {
		    e1.printStackTrace();
		}
		
	}
	
	public void run() {
		
		// Refer to an existing transformation via URI
		URI transformationURI = URI.createFileURI(transPath);
	
		// create executor for the given transformation
		TransformationExecutor executor = new TransformationExecutor(transformationURI);
	
		// Load Provider model as QVTo input model
		EXTLibraryPackage.eINSTANCE.eClass();

		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> map = reg
				.getExtensionToFactoryMap();
		map.put("extlibrary", new XMIResourceFactoryImpl());
		
		
		ResourceSet resSet = new ResourceSetImpl();
		Resource resource = resSet.getResource(
				URI.createFileURI(libPath), true);
		
		ModelExtent modelInput = new BasicModelExtent(
				resource.getContents());
		
		
		// Create an empty extent to catch the uml model output
		UMLPackage.eINSTANCE.eClass();
		
		Resource.Factory.Registry umlReg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> umlMap = umlReg
				.getExtensionToFactoryMap();
		umlMap.put("uml", new XMIResourceFactoryImpl());

		ModelExtent umlOutput = new BasicModelExtent();
		
		// setup the execution environment details ->
		// configuration properties, logger, monitor object etc.
		ExecutionContextImpl context = new ExecutionContextImpl();
		// Configure context to print logs called from the QVTo specification
		OutputStreamWriter outStream = new OutputStreamWriter(System.out);
		Log log = new WriterLog(outStream);
		context.setLog(log);

		// Execute transformation
		ExecutionDiagnostic result = executor.execute(context, modelInput, umlOutput);

		// check the result for success
		if (result.getSeverity() == Diagnostic.OK) {
			// The output objects got captured in the output extent
			List<EObject> outObjects = umlOutput.getContents();
			System.out.println("Transformation completed");
			// Load Document model to be saved from the QVTo output
			ResourceSet documentResSet = new ResourceSetImpl();
			
			Resource documentResource = documentResSet.createResource(URI
					.createFileURI(umlPath));
			// Add QVTo output to resource
			documentResource.getContents().addAll(outObjects);
			// Save resource
			try {
				documentResource.save(Collections.emptyMap());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			IStatus status = BasicDiagnostic.toIStatus(result);
			System.out.println(status.getMessage());
			for (IStatus s : status.getChildren())
				System.out.println(s.getMessage());
			// Activator.getDefault().getLog().log(status);
		}

		
		
		
	}
	
}
